@@ -130,12 +130,13 @@ ldap_def = {
       # Sun Directory Server
       'isMemberOf',
       # MS Active Directory
+      'nTSecurityDescriptor',
       'tokenGroups','tokenGroupsGlobalAndUniversal','tokenGroupsNoGCAcceptable',
       'mS-DS-CreatorSID','primaryGroupToken','canonicalName','fromEntry',
       'sDRightsEffective','msDS-Approx-Immed-Subordinates','msDS-KeyVersionNumber',
 #      'msDS-ReplAttributeMetaData',
 #      'lastLogonTimestamp','lockoutTime',
-#      'allowedAttributes','allowedAttributesEffective','allowedChildClasses','allowedChildClassesEffective',
+      'allowedAttributes','allowedAttributesEffective','allowedChildClasses','allowedChildClassesEffective',
       # X.500 DSAs
       'administrativeRole',
     ],
@@ -199,13 +200,7 @@ ldap_def = {
     login_default_mech='',
 
     # Attributes which should be present in attribute select list of advanced search form
-    search_attrs=[
-      'cn','mail','sn','givenName',
-      'o','ou','departmentNumber','employeeNumber',
-      'telephoneNumber','homePhone','mobile',
-      'l','streetAddress','uid','uidNumber','description',
-      'objectClass','entryUUID',
-    ],
+    search_attrs=[],
 
     # There are some situations where this client just wants to get the
     # attributes of an entry and not the data itself for saving bandwidth.
@@ -382,6 +377,15 @@ ldap_def = {
     ssl_valid_dn=r'^/S=Stroeder/G=Michael/CN=Michael Stroeder/Email=michael@stroeder.com$',
     ssl_valid_idn=r'^/C=ZA/ST=Western Cape/L=Durbanville/O=Thawte/OU=Certificate Services/CN=Personal Freemail RSA 1999\.9\.16$',
     session_track_control=1,
+    searchform_search_root_url=u'ldap:///dc=stroeder,dc=de??sub?(&(|(ou:dn:=Bizness)(ou:dn:=Kultur))(objectClass=organization)(hasSubordinates=TRUE))',
+    search_attrs=[
+      'cn','mail','sn','givenName','personalTitle','businessTitle',
+      'o','ou','departmentNumber','employeeNumber',
+      'telephoneNumber','homePhone','mobile',
+      'c','st','l','streetAddress','roomNumber',
+      'uid','uidNumber','description',
+      'objectClass','entryUUID','organizationalStatus',
+    ],
     read_template={
       # 'object class':'pathname of HTML template file'
       'germanBankArrangement':os.path.join(web2ldapcnf.templates_dir,'read_germanBankArrangement.html'),
@@ -434,14 +438,14 @@ ldap_def = {
   'ldap:///cn=accesslog': Web2LDAPConfig(
     description=u'OpenLDAP accesslog',
     search_tdtemplate= {
-      'auditsearch':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s &rArr; %(reqresult)s<br>Search %(reqscope)s: %(reqdn)s<br>%(reqfilter)s',
-      'auditbind':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s &rArr; %(reqresult)s: %(reqsession)s<br>%(reqdn)s',
-      'auditadd':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s &rArr; %(reqresult)s<br>Entry %(reqdn)s added<br>by %(reqauthzid)s',
-      'auditmodify':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s &rArr; %(reqresult)s<br>Entry %(reqdn)s modified<br>by %(reqauthzid)s',
-      'auditmodrdn':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s &rArr; %(reqresult)s<br>Entry %(reqdn)s renamed to %(reqnewrdn)s,%(reqnewsuperior)s<br>by %(reqauthzid)s',
-      'auditobject':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s &rArr; %(reqresult)s<br>by %(reqauthzid)s',
-      'auditdelete':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s &rArr; %(reqresult)s<br>Entry %(reqdn)s deleted<br>by %(reqauthzid)s',
-      'auditabandon':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s &rArr; %(reqresult)s<br>Abandonded %(reqId)s by %(reqauthzid)s',
+      'auditsearch':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s <br>&rArr; %(reqresult)s: %(reqMessage)s<br>Search %(reqscope)s: %(reqdn)s<br>%(reqfilter)s',
+      'auditbind':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s <br>&rArr; %(reqresult)s: %(reqMessage)s: %(reqsession)s<br>%(reqdn)s',
+      'auditadd':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s <br>&rArr; %(reqresult)s: %(reqMessage)s<br>Entry %(reqdn)s added<br>by %(reqauthzid)s',
+      'auditmodify':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s <br>&rArr; %(reqresult)s: %(reqMessage)s<br>Entry %(reqdn)s modified<br>by %(reqauthzid)s',
+      'auditmodrdn':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s <br>&rArr; %(reqresult)s: %(reqMessage)s<br>Entry %(reqdn)s renamed to %(reqnewrdn)s,%(reqnewsuperior)s<br>by %(reqauthzid)s',
+      'auditobject':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s <br>&rArr; %(reqresult)s: %(reqMessage)s<br>by %(reqauthzid)s',
+      'auditdelete':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s <br>&rArr; %(reqresult)s: %(reqMessage)s<br>Entry %(reqdn)s deleted<br>by %(reqauthzid)s',
+      'auditabandon':r'<strong>%(reqtype)s</strong> %(reqstart)s session %(reqSession)s <br>&rArr; %(reqresult)s: %(reqMessage)s<br>Abandonded %(reqId)s by %(reqauthzid)s',
     },
     searchform_template= {
       u'_':os.path.join(web2ldapcnf.templates_dir,'searchform_accesslog.html'),
@@ -477,7 +481,8 @@ ldap_def = {
       # for OpenLDAP
       'monitorOperation':r'%(cn)s operations: %(monitorOpCompleted)s of %(monitorOpInitiated)s completed',
       'monitoredObject':r"""%(cn)s: %(monitoredInfo)s<br>
-         %(monitorTimestamp)s %(namingContexts)s %(labeledURI)s
+         %(monitorTimestamp)s %(namingContexts)s %(labeledURI)s<br>
+         %(seeAlso)s
         """,
       'monitorContainer':r'%(cn)s - %(description)s',
       'monitorCounterObject':r'%(cn)s: %(monitorCounter)s',
