@@ -6,13 +6,8 @@ ldapsession.py - higher-level class for handling LDAP connections
 This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
-
-$Id: ldapsession.py,v 1.378 2013/01/13 15:47:47 michael Exp $
 """
 
-__version__ = '$Revision: 1.378 $'.split(' ')[1]
-
-
 import sys,socket,time,types,codecs,ldap,ldap.cidict,ldap.resiter,ldaputil.base,ldaputil.schema
 
 try:
@@ -23,8 +18,9 @@ except ImportError:
 from ldap.ldapobject import ReconnectLDAPObject
 from ldapurl import LDAPUrl,LDAPUrlExtension
 from ldap.schema.models import DITStructureRule
+from ldaputil.base import escape_ldap_filter_chars
 from ldaputil.extldapurl import ExtendedLDAPUrl
-from ldaputil.controls import PreReadControl,PostReadControl
+from ldaputil.controls import PreReadControl,PostReadControl,SearchNoOpControl
 
 from ldap.controls import ValueLessRequestControl,AssertionControl
 # Authorization Identity Request and Response Controls (see RFC 3829)
@@ -118,6 +114,10 @@ USER_ENTRY_ATTRIBUTES = (
 
 READ_CACHE_EXPIRE = 120
 
+LDAPLimitErrors = (ldap.TIMEOUT,ldap.TIMELIMIT_EXCEEDED,ldap.SIZELIMIT_EXCEEDED,ldap.ADMINLIMIT_EXCEEDED)
+
+COUNT_TIMEOUT = 5.0
+
 
 class MyLDAPObject(ReconnectLDAPObject,ldap.resiter.ResultProcessor):
 
@@ -241,6 +241,31 @@ class MyLDAPObject(ReconnectLDAPObject,ldap.resiter.ResultProcessor):
       self._cache[cache_key_args] = (current_time+self.cache_time,result)
     return result # search_ext_s()
 
+  def noop_search_st(self,base,scope=ldap.SCOPE_SUBTREE,filterstr='(objectClass=*)',timeout=-1):
+    try:
+      msg_id = self.search_ext(
+        base,
+        scope,
+        filterstr=filterstr,
+        attrlist=['1.1'],
+        timeout=timeout,
+        serverctrls=[SearchNoOpControl()],
+      )
+      _,_,_,search_response_ctrls = self.result3(msg_id,all=1,timeout=timeout)
+    except LDAPLimitErrors,e:
+      self.abandon(msg_id)
+      raise e
+    else:
+      noop_srch_ctrl = [
+        c
+        for c in search_response_ctrls
+        if c.controlType==SearchNoOpControl.controlType
+      ]
+      if noop_srch_ctrl:
+        return noop_srch_ctrl[0].numSearchResults,noop_srch_ctrl[0].numSearchContinuations
+      else:
+        return (None,None)
+
   def get_cache_hit_ratio(self):
     """
     Returns percentage of cache hit ratio
@@ -293,10 +318,14 @@ class PWD_CHANGE_AFTER_RESET(PasswordPolicyException):
   pass
 
 class PWD_EXPIRATION_WARNING(PasswordPolicyException):
-  pass
+  def __init__(self,who=None,desc=None,timeBeforeExpiration=None):
+    PasswordPolicyException.__init__(self,who,desc)
+    self.timeBeforeExpiration = timeBeforeExpiration
 
 class PWD_EXPIRED(PasswordPolicyException):
-  pass
+  def __init__(self,who=None,desc=None,graceAuthNsRemaining=None):
+    PasswordPolicyException.__init__(self,who,desc)
+    self.graceAuthNsRemaining = graceAuthNsRemaining
 
 class USERNAME_NOT_FOUND(LDAPSessionException):
   """
@@ -576,15 +605,15 @@ class LDAPSession:
                          'msDS-Approx-Immed-Subordinates')
     # First try to read operational attributes from entry itself
     # which might indicate whether there are subordinate entries
-    result_ldap = self.l.search_ext_s(
+    ldap_result = self.l.search_ext_s(
       self.uc_encode(dn)[0],
       ldap.SCOPE_BASE,'(objectClass=*)',
       subordinate_attrs,
       timeout=self.timeout
     )
     hasSubordinates = None; numSubordinates = None; numAllSubordinates = None
-    if result_ldap:
-      entry = ldap.cidict.cidict(result_ldap[0][1])
+    if ldap_result:
+      entry = ldap.cidict.cidict(ldap_result[0][1])
       for a in ('subordinateCount','numSubordinates','msDS-Approx-Immed-Subordinates'):
         try:
           numSubordinates = int(entry[a][0])
@@ -595,7 +624,17 @@ class LDAPSession:
       try:
         numAllSubordinates = int(entry['numAllSubordinates'][0])
       except KeyError:
-        pass
+        if numSubordinates!=None:
+          ldap_result = self.l.search_ext_s(
+            self.uc_encode(dn)[0],
+            ldap.SCOPE_SUBTREE,
+            '(numSubordinates>=1)',
+            attrlist=['numSubordinates'],
+            timeout=COUNT_TIMEOUT
+          )
+          numAllSubordinates = 0
+          for ldap_dn,ldap_entry in ldap_result:
+            numAllSubordinates += int(ldap_entry['numSubordinates'][0])
       try:
         hasSubordinates = entry['hasSubordinates'][0].upper()=='TRUE'
       except KeyError:
@@ -610,11 +649,32 @@ class LDAPSession:
         ldap.SCOPE_ONELEVEL,'(objectClass=*)',
         ['objectClass'],self.onlyAttrTypes,timeout=self.timeout,sizelimit=1
       )
-      result_ldap = (None,None)
-      while result_ldap==(None,None):
-        result_ldap = self.l.result(ldap_msgid,0,self.timeout)
+      ldap_result = (None,None)
+      while ldap_result==(None,None):
+        ldap_result = self.l.result(ldap_msgid,0,self.timeout)
       self.l.abandon(ldap_msgid)
-      hasSubordinates = len(result_ldap)>0
+      hasSubordinates = len(ldap_result)>0
+
+    if SearchNoOpControl.controlType in self.rootDSE.get('supportedControl',[]):
+      if not numSubordinates:
+        try:
+          numSubordinates,_ = self.l.noop_search_st(
+            self.uc_encode(dn)[0],
+            ldap.SCOPE_ONELEVEL,
+            timeout=COUNT_TIMEOUT,
+          )
+        except LDAPLimitErrors,e:
+          pass
+      if not numAllSubordinates:
+        try:
+          numAllSubordinates,_ = self.l.noop_search_st(
+            self.uc_encode(dn)[0],
+            ldap.SCOPE_SUBTREE,
+            timeout=COUNT_TIMEOUT,
+          )
+        except LDAPLimitErrors,e:
+          pass
+
     return (hasSubordinates,numSubordinates,numAllSubordinates)
 
   def getObjectClasses(self,dn):
@@ -909,7 +969,8 @@ class LDAPSession:
     elif ldaputil.base.is_dn(username):
       return ldaputil.base.normalize_dn(username)
     else:
-      searchfilter = filtertemplate.replace('%s',username)
+      username_filter_escaped = escape_ldap_filter_chars(username)
+      searchfilter = filtertemplate.replace('%s',username_filter_escaped)
       if searchroot is None:
         searchroot = u''
       # Try to find a unique entry with filtertemplate
@@ -1035,8 +1096,11 @@ class LDAPSession:
     # Determine identity by sending LDAPv3 Who Am I? extended operation
     try:
       whoami = unicode(self.l.whoami_s(),self.charset)
-    except ldap.LDAPError:
-      self.who = u'u:%s' % (who)
+    except ldap.LDAPError,e:
+      if who:
+        self.who = u'u:%s' % (who)
+      else:
+        self.who = None
     else:
       if whoami.startswith('dn:'):
         self.who = whoami[3:]
@@ -1107,11 +1171,13 @@ class LDAPSession:
           raise PWD_EXPIRATION_WARNING(
             who=self.who.encode(self.charset),
             desc='Password will expire in %d seconds!' % (ppolicy_ctrl.timeBeforeExpiration),
+            timeBeforeExpiration=ppolicy_ctrl.timeBeforeExpiration,
           )
         elif ppolicy_ctrl.graceAuthNsRemaining!=None:
           raise PWD_EXPIRED(
             who=self.who.encode(self.charset),
             desc='Password expired! %d grace logins left.' % (ppolicy_ctrl.graceAuthNsRemaining),
+            graceAuthNsRemaining=ppolicy_ctrl.graceAuthNsRemaining,
           )
 
     return # bind()
