@@ -7,14 +7,11 @@ This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
 
-$Id: msHTTPHandler.py,v 1.29 2012/12/16 15:53:30 michael Exp $
-
 The HTTPHandlerClass is derived from
 class SimpleHTTPServer.SimpleHTTPRequestHandler.
 """
 
-__version__ = '$Revision: 1.29 $'.split(' ')[1]
-
+__version__ = '1.1.37'
 
 import sys,os,posixpath,stat,socket,string,time,SimpleHTTPServer,urllib,msbase
 
@@ -154,6 +151,7 @@ class HTTPHandlerClass(SimpleHTTPServer.SimpleHTTPRequestHandler):
       ('Accept-Language','HTTP_ACCEPT_LANGUAGE'),
       ('Referer','HTTP_REFERER'),
       ('Connection','HTTP_CONNECTION'),
+      ('Cookie','HTTP_COOKIE'),
     ]:
       http_header_value = self.headers.getheader(envitem[0])
       if http_header_value:
