@@ -5,8 +5,6 @@ x509.py - X.509 certificate objects
 This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
-
-$Id: x509.py,v 1.16 2013/02/05 20:12:19 michael Exp $
 """
 
 # Python standard lib
@@ -14,7 +12,7 @@ import sys, string, base64, hashlib
 # Pisces
 from pisces import asn1
 # mspki itself
-import utctime, util, x500, asn1helper
+import utctime, util, x500, x509v3, asn1helper
 
 class Attribute(asn1.ASN1Object):
   """
@@ -128,31 +126,31 @@ class Certificate(X509SignedObject):
     if hasattr(self.tbsCertificate[0],'tag') and \
        self.tbsCertificate[0].tag==0:
       # no version number present
-      self.__version__   = Version(self.tbsCertificate[0].val)
-      self.__tbsoffset__ = 1
+      self._version   = Version(self.tbsCertificate[0].val)
+      self._tbsoffset = 1
     else:
       # version number present and encoded in contextual object
-      self.__version__   = Version(None)
-      self.__tbsoffset__ = 0
+      self._version   = Version(None)
+      self._tbsoffset = 0
 
   def version(self):
     """X.509 certificate version number as integer"""
-    return self.__version__
+    return self._version
 
   def serialNumber(self):
     """Certificate's serial number as long integer"""
-    return CertificateSerialNumber(self.tbsCertificate[self.__tbsoffset__+0])
+    return CertificateSerialNumber(self.tbsCertificate[self._tbsoffset+0])
 
   def signature(self,oids=None):
     """Certificate's signature"""
     if oids:
-      return asn1helper.GetOIDDescription(self.tbsCertificate[self.__tbsoffset__+1].val[0],oids)
+      return asn1helper.GetOIDDescription(self.tbsCertificate[self._tbsoffset+1].val[0],oids)
     else:
-      return self.tbsCertificate[self.__tbsoffset__+1].val[0]
+      return self.tbsCertificate[self._tbsoffset+1].val[0]
 
   def issuer(self):
     """Issuer's distinguished name"""
-    return x500.Name(self.tbsCertificate[self.__tbsoffset__+2])
+    return x500.Name(self.tbsCertificate[self._tbsoffset+2])
 
   def validity(self):
     """
@@ -162,21 +160,21 @@ class Certificate(X509SignedObject):
     containing UTCTime of begin and end of validity period.
     """
     return (
-      utctime.UTCTime(self.tbsCertificate[self.__tbsoffset__+3][0].val),
-      utctime.UTCTime(self.tbsCertificate[self.__tbsoffset__+3][1].val)
+      utctime.UTCTime(self.tbsCertificate[self._tbsoffset+3][0].val),
+      utctime.UTCTime(self.tbsCertificate[self._tbsoffset+3][1].val)
     )
 
   def subject(self):
     """Subject's distinguished name"""
-    return x500.Name(self.tbsCertificate[self.__tbsoffset__+4])
+    return x500.Name(self.tbsCertificate[self._tbsoffset+4])
 
   def subjectPublicKeyInfo(self,oids=None):
     """Subject's public key"""
     if oids:
-      alg = asn1helper.GetOIDDescription(self.tbsCertificate[self.__tbsoffset__+5].val[0].val[0],oids)
+      alg = asn1helper.GetOIDDescription(self.tbsCertificate[self._tbsoffset+5].val[0].val[0],oids)
     else:
-      alg = self.tbsCertificate[self.__tbsoffset__+5].val[0].val[0]
-    modulus, publicExponent = asn1.parse(self.tbsCertificate[self.__tbsoffset__+5].val[1].val)
+      alg = self.tbsCertificate[self._tbsoffset+5].val[0].val[0]
+    modulus, publicExponent = asn1.parse(self.tbsCertificate[self._tbsoffset+5].val[1].val)
     return (alg, modulus, publicExponent)
 
   def fingerprint(self,digest_algo,delimiter=':'):
@@ -185,7 +183,7 @@ class Certificate(X509SignedObject):
 
   def issuerUniqueID(self):
     """Get subjectUniqueID (tag 1)"""
-    for i in range(self.__tbsoffset__+6,len(self.tbsCertificate)):
+    for i in range(self._tbsoffset+6,len(self.tbsCertificate)):
       if isinstance(self.tbsCertificate[i],asn1.Contextual):
         if self.tbsCertificate[i].tag==1:
           return self.tbsCertificate[i]
@@ -193,7 +191,7 @@ class Certificate(X509SignedObject):
 
   def subjectUniqueID(self):
     """Get subjectUniqueID (tag 2)"""
-    for i in range(self.__tbsoffset__+6,len(self.tbsCertificate)):
+    for i in range(self._tbsoffset+6,len(self.tbsCertificate)):
       if isinstance(self.tbsCertificate[i],asn1.Contextual):
         if self.tbsCertificate[i].tag==2:
           return self.tbsCertificate[i]
@@ -274,39 +272,39 @@ class CRL(X509SignedObject):
     # Try to determine if optional version field is present
     # FIX ME!!! This is a pretty ugly hack!
     if isinstance(self.tbsCertList[0],asn1.Sequence):
-      self.__version__   = Version(None)
-      self.__tbsoffset__ = 0
+      self._version   = Version(None)
+      self._tbsoffset = 0
     else:
-      self.__version__   = Version(self.tbsCertList[0])
-      self.__tbsoffset__ = 1
+      self._version   = Version(self.tbsCertList[0])
+      self._tbsoffset = 1
 
   def version(self):
     """X.509 CRL version number as integer"""
-    return self.__version__
+    return self._version
 
   def signature(self,oids=None):
     """Certificate's signature algorithm"""
     if oids:
       return asn1helper.GetOIDDescription(
-        self.tbsCertList[self.__tbsoffset__+0].val[0],
+        self.tbsCertList[self._tbsoffset+0].val[0],
         oids
       )
     else:
-      return self.tbsCertList[self.__tbsoffset__+0].val[0]
+      return self.tbsCertList[self._tbsoffset+0].val[0]
 
   def issuer(self):
     """Issuer's distinguished name"""
-    return x500.Name(self.tbsCertList[self.__tbsoffset__+1])
+    return x500.Name(self.tbsCertList[self._tbsoffset+1])
 
   def thisUpdate(self):
     """Returns time tuple of thisUpdate"""
-    return utctime.UTCTime(self.tbsCertList[self.__tbsoffset__+2].val)
+    return utctime.UTCTime(self.tbsCertList[self._tbsoffset+2].val)
 
   def nextUpdate(self):
     """Returns utctime.UTCTime of nextUpdate if present, None else"""
-    if isinstance(self.tbsCertList[self.__tbsoffset__+3],asn1.UTCTime) or \
-       isinstance(self.tbsCertList[self.__tbsoffset__+3],asn1.GeneralizedTime):
-      return utctime.UTCTime(self.tbsCertList[self.__tbsoffset__+3].val)
+    if isinstance(self.tbsCertList[self._tbsoffset+3],asn1.UTCTime) or \
+       isinstance(self.tbsCertList[self._tbsoffset+3],asn1.GeneralizedTime):
+      return utctime.UTCTime(self.tbsCertList[self._tbsoffset+3].val)
     else:
       return None
 
@@ -323,9 +321,16 @@ class CRL(X509SignedObject):
     )
     """
     revokeList = []
-    if len (self.tbsCertList)>=self.__tbsoffset__+4+1 and \
-       (not hasattr(self.tbsCertList[self.__tbsoffset__+4],'tag') or self.tbsCertList[self.__tbsoffset__+4].tag!=0):
-      for i in self.tbsCertList[self.__tbsoffset__+4].val:
+    # Check whether nextUpdate is there
+    if isinstance(self.tbsCertList[self._tbsoffset+3],asn1.UTCTime) or \
+       isinstance(self.tbsCertList[self._tbsoffset+3],asn1.GeneralizedTime):
+      crl_offset = 4
+    else:
+      # nextUpdate is missing
+      crl_offset = 3
+    if len (self.tbsCertList)>=self._tbsoffset+crl_offset+1 and \
+       (not hasattr(self.tbsCertList[self._tbsoffset+crl_offset],'tag') or self.tbsCertList[self._tbsoffset+crl_offset].tag!=0):
+      for i in self.tbsCertList[self._tbsoffset+crl_offset].val:
         i_len = len(i)
         if i_len in [2,3]:
           userCertificate = i[0]
@@ -336,43 +341,8 @@ class CRL(X509SignedObject):
             "Item in revokedCertificates list has invalid length (%d)." % (i_len)
           )
         if i_len==3:
-          crlEntryExtensions = i[2]
+          crlEntryExtensions = x509v3.Extensions(i[2])
         else:
           crlEntryExtensions = None
         revokeList.append((userCertificate,revocationDate,crlEntryExtensions))
     return revokeList
-
-  def as_text(self,oids=None):
-    revokeList = self.revokedCertificates()
-    if revokeList:
-      revokeList_str = string.join(
-        map(
-          lambda x:
-            '    Serial Number: %d\n        Revocation Date: %s' % (x[0],x[1]),
-          revokeList
-        ),
-        '\n'
-      )
-    else:
-      revokeList_str = 'None'
-    return """Certificate Revocation List (CRL):
-        Version %X
-        Signature Algorithm: %s
-        Issuer: %s
-        Last Update: %s
-        Next Update: %s
-Revoked Certificates: %d
-%s
-    Signature Algorithm: %s
-%s
-""" % (
-      self.version(),
-      self.signature(oids),
-      self.issuer().__str__(oids),
-      self.thisUpdate(),
-      self.nextUpdate(),
-      len(revokeList),
-      revokeList_str,
-      self.signatureAlgorithm(oids),
-      util.HexString(self.signatureValue(),wrap=64,indent=8)
-    )
