@@ -10,14 +10,12 @@ see http://www.web2ldap.de for details
 This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
-
-$Id: form.py,v 1.120 2012/10/24 17:47:16 michael Exp $
 """
 
 import urllib,codecs,re,ldif,ldapurl,ldap.schema,msgzip,ldaputil.base, \
        pyweblib.forms,ldapsession
 
-import w2lapp.cnf,w2lapp.core,w2lapp.gui,w2lapp.passwd,w2lapp.searchform,w2lapp.ldapparams
+import w2lapp.cnf,w2lapp.core,w2lapp.gui,w2lapp.passwd,w2lapp.searchform,w2lapp.ldapparams,w2lapp.session
 
 # OID description dictionary from configuration directory
 from ldapoidreg import oid as oid_desc_reg
@@ -90,6 +88,13 @@ class Web2LDAPForm(pyweblib.forms.Form):
     self.addCommandFields()
 
   def addGeneralFields(self):
+    self.addField(pyweblib.forms.Input(
+      'delsid',
+      u'Old SID to be deleted',
+      w2lapp.session.session.session_id_len,
+      1,
+      w2lapp.session.session.session_id_re.pattern
+    ))
     self.addField(pyweblib.forms.Input('who',u'Bind DN/AuthcID',1000,1,u'.*',size=40))
     self.addField(pyweblib.forms.Input('cred',u'with Password',200,1,u'.*',size=15))
     self.addField(pyweblib.forms.Select('login_authzid_prefix',u'SASL AuthzID',1,options=[('','- no prefix -'),('u:','user-ID'),('dn:','DN')],default=''))
@@ -99,7 +104,11 @@ class Web2LDAPForm(pyweblib.forms.Form):
     self.addField(pyweblib.forms.Input('ldapurl',u'LDAP Url',1024,1,'[ ]*ldap(|i|s)://.*',size=30))
     self.addField(pyweblib.forms.Input('host',u'Host:Port',255,1,'(%s|[a-zA-Z0-9/._-]+)' % w2lapp.gui.host_pattern,size=30))
     self.addField(DistinguishedNameInput('dn','Distinguished Name'))
-    self.addField(pyweblib.forms.Select('scope','Scope',1,options=[('0','Base'),('1','One level'),('2','Sub tree')],default='2'))
+    self.addField(pyweblib.forms.Select(
+      'scope','Scope',1,
+      options=w2lapp.searchform.SEARCH_SCOPE_OPTIONS,
+      default=w2lapp.searchform.SEARCH_SCOPE_STR_SUBTREE),
+    )
     self.addField(DistinguishedNameInput('login_search_root','Login search root'))
     self.addField(pyweblib.forms.Input('login_filterstr',u'Login search filter string',300,1,'.*'))
     self.addField(pyweblib.forms.Select(
@@ -200,9 +209,9 @@ class Web2LDAPNullForm(Web2LDAPForm):
 class Web2LDAPForm_searchform(Web2LDAPForm):
 
   def addCommandFields(self):
-    self.addField(pyweblib.forms.Select('search_submit',u'Search form submit button',1,options=[u'Search',u'More',u'Fewer',u'All Attrs'],default=None))
+    self.addField(pyweblib.forms.Select('search_submit',u'',1,options=[u'Search',u'More',u'Fewer',u'All Attrs'],default=None))
+    self.addField(pyweblib.forms.Input('search_submit',u'Search form submit button',6,1,'(Search|[+-][0-9]+)'))
     self.addField(pyweblib.forms.Select('searchform_mode',u'Search form mode',1,options=[(u'base',u'Base'),(u'adv',u'Advanced'),(u'exp',u'Expert')],default=u'base'))
-    self.addField(pyweblib.forms.Field('search_params',u'Number of search parameters',2,1,'[0-9]+'))
     self.addField(DistinguishedNameInput('search_root','Search root'))
     self.addField(pyweblib.forms.Input('filterstr',u'Search filter string',1200,1,'.*'))
     self.addField(pyweblib.forms.Input('searchform_template',u'Search form template name',60,w2lapp.cnf.misc.max_searchparams,u'[a-zA-Z0-9. ()-]+'))
@@ -237,14 +246,15 @@ class Web2LDAPForm_searchform(Web2LDAPForm):
     self.addField(InclOpAttrsCheckbox('search_opattrs',u'Request operational attributes',default="yes",checked=0))
     self.addField(pyweblib.forms.Select('search_mode',u'Search Mode',1,options=[ur'(&%s)',ur'(|%s)']))
     self.addField(pyweblib.forms.Input('search_attr',u'Attribute(s) to be searched',1000,w2lapp.cnf.misc.max_searchparams,ur'[\w,_;-]+'))
+    self.addField(pyweblib.forms.Input('search_mr',u'Matching Rule',1000,w2lapp.cnf.misc.max_searchparams,ur'[\w,_;-]+'))
     self.addField(pyweblib.forms.Select('search_option',u'Search option',w2lapp.cnf.misc.max_searchparams,options=w2lapp.searchform.search_options))
-    self.addField(pyweblib.forms.Input('search_string',u'Search string',100,w2lapp.cnf.misc.max_searchparams,u'.*',size=50))
-    self.addField(pyweblib.forms.Input('search_attrs',u'Attributes to be read',1000,1,ur'[*+0-9.\w,_;-]+'))
+    self.addField(pyweblib.forms.Input('search_string',u'Search string',100,w2lapp.cnf.misc.max_searchparams,u'.*',size=60))
+    self.addField(pyweblib.forms.Input('search_attrs',u'Attributes to be read',1000,1,ur'[@*+0-9.\w,_;-]+'))
 
 class Web2LDAPForm_search(Web2LDAPForm_searchform):
   def addCommandFields(self):
     Web2LDAPForm_searchform.addCommandFields(self)
-    self.addField(pyweblib.forms.Input('filterstr',u'Search filter string',1200,1,u'.*'))
+    self.addField(pyweblib.forms.Input('filterstr',u'Search filter string',1200,20,'.*'))
     self.addField(DistinguishedNameInput('search_root','Search root'))
     self.addField(pyweblib.forms.Input('search_resminindex',u'Minimum index of search results',10,1,u'[0-9]+'))
     self.addField(pyweblib.forms.Input('search_resnumber',u'Number of results to display',3,1,u'[0-9]+'))
@@ -318,16 +328,16 @@ class Web2LDAPForm_input(Web2LDAPForm):
 
 class Web2LDAPForm_add(Web2LDAPForm_input):
   def addCommandFields(self):
+    Web2LDAPForm_input.addCommandFields(self)
     self.addField(pyweblib.forms.Input('add_rdn',u'RDN of new entry',255,1,u'.*',size=50))
     self.addField(pyweblib.forms.Input('add_basedn',u'Base DN of new entry',1024,1,u'.*',size=50))
-    Web2LDAPForm_input.addCommandFields(self)
 
 class Web2LDAPForm_modify(Web2LDAPForm_input):
   def addCommandFields(self):
+    Web2LDAPForm_input.addCommandFields(self)
     self.addField(AttributeType('in_oldattrtypes',u'Old attribute types',w2lapp.cnf.misc.input_maxattrs))
     self.addField(AttributeType('in_wrtattroids',u'Writeable attribute types',w2lapp.cnf.misc.input_maxattrs))
     self.addField(pyweblib.forms.Input('in_assertion',u'Assertion filter string',2000,1,'.*',required=1))
-    Web2LDAPForm_input.addCommandFields(self)
 
 class Web2LDAPForm_dds(Web2LDAPForm):
   def addCommandFields(self):
@@ -410,7 +420,7 @@ class Web2LDAPForm_read(Web2LDAPForm):
     self.addField(pyweblib.forms.Input('read_attrindex',u'Read attribute',255,1,u'[0-9]+'))
     self.addField(pyweblib.forms.Input('read_attrmimetype',u'MIME type',255,1,u'[\w.-]+/[\w.-]+'))
     self.addField(pyweblib.forms.Select('read_output',u'Read output format',1,options=['table','vcard','template'],default='template'))
-    self.addField(pyweblib.forms.Input('search_attrs',u'Attributes to be read',1000,1,ur'[*+\w,_;-]+'))
+    self.addField(pyweblib.forms.Input('search_attrs',u'Attributes to be read',1000,1,ur'[@*+0-9.\w,_;-]+'))
     self.addField(pyweblib.forms.Input('read_expandattr',u'Attributes to be read',1000,1,ur'[*+\w,_;-]+'))
 
 class Web2LDAPForm_groupadm(Web2LDAPForm):
@@ -519,7 +529,7 @@ class OIDInput(pyweblib.forms.Input):
       self,name,text,
       512,1,u'[a-zA-Z0-9_.;*-]+',
       default=default,
-      required=0,size=25
+      required=0,size=30
     )
 
 
