@@ -10,13 +10,11 @@ see http://www.web2ldap.de for details
 This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
-
-$Id: gui.py,v 1.276 2013/02/08 18:40:10 michael Exp $
 """
 
-import os,urllib,ldap, \
+import os,urllib,ldap,ldapurl, \
        ldaputil,pyweblib.forms,pyweblib.httphelper,msbase, \
-       w2lapp.core,w2lapp.cnf,w2lapp.schema.syntaxes
+       w2lapp.core,w2lapp.cnf,w2lapp.schema.syntaxes,w2lapp.searchform
 
 from ldapurl import LDAPUrl,LDAPUrlExtension,isLDAPUrl
 from ldap.filter import escape_filter_chars
@@ -110,6 +108,11 @@ def LDAPError2ErrMsg(e,form,charset='utf-8',template='{error_msg}<br>{matched_dn
         unicode(e.args[0]['desc'],charset),
         unicode(e.args[0].get('info',''),charset)
       ))
+    except UnicodeDecodeError:
+      ErrMsg = u':\n'.join((
+        unicode(e.args[0]['desc'],charset),
+        unicode(repr(e.args[0].get('info','')),charset)
+      ))
     except TypeError:
       try:
         ErrMsg = u':\n'.join((
@@ -129,7 +132,7 @@ def LDAPError2ErrMsg(e,form,charset='utf-8',template='{error_msg}<br>{matched_dn
 
   # Add matchedDN to error message HTML if needed
   if matched_dn:
-    matched_dn_html = '<br>Matched DN: %s' % (form.utf2display(matched_dn.decode(charset)))
+    matched_dn_html = '<br>Matched DN: %s' % (form.utf2display(matched_dn))
   else:
     matched_dn_html = ''
 
@@ -139,6 +142,37 @@ def LDAPError2ErrMsg(e,form,charset='utf-8',template='{error_msg}<br>{matched_dn
   )
 
 
+def ts2repr(time_divisors,ts_sep,ts_value):
+  rest = long(ts_value)
+  result = []
+  for desc,divisor in time_divisors:
+    mult = rest / divisor
+    rest = rest % divisor
+    if mult>0:
+      result.append(u'%d %s' % (mult,desc))
+    if rest==0:
+      break
+  return ts_sep.join(result)
+
+
+def repr2ts(time_divisors,ts_sep,value):
+  l1 = [ v.strip().split(u' ') for v in value.split(ts_sep) ]
+  l2 = [
+    (int(v),d.strip())
+    for v,d in l1
+  ]
+  time_divisors_dict = dict(time_divisors)
+  result = 0
+  for value,desc in l2:
+    try:
+      result += value*time_divisors_dict[desc]
+    except KeyError:
+      raise ValueError
+    else:
+      del time_divisors_dict[desc]
+  return result
+
+
 def DisplayDN(sid,form,ls,dn,commandbutton=0,active_ldapurl=0,bindas_button=False):
   """Display a DN as LDAP URL with or without button"""
   assert type(dn)==UnicodeType, "Argument 'dn' must be UnicodeType"
@@ -391,17 +425,29 @@ def MainMenu(sid,form,ls,dn):
       cl.append(
         form.applAnchor(
           'search','Up',sid,
-          [
+          (
             ('dn',ParentDN(dn)),
-            ('scope',str(ldap.SCOPE_ONELEVEL)),
-          ],
+            ('scope',w2lapp.searchform.SEARCH_SCOPE_STR_ONELEVEL),
+            ('filterstr',u'(objectClass=*)'),
+          ),
           title=u'Go up one level',
         )
       )
 
     cl.extend((
-      form.applAnchor('search','Down',sid,(('dn',dn),('scope',unicode(str(ldap.SCOPE_ONELEVEL)))),title=u'Descend into tree from here'),
-      form.applAnchor('searchform','Search',sid,[('dn',dn)],title=u'Enter search criteria in input form'),
+      form.applAnchor(
+        'search','Down',sid,
+        (
+          ('dn',dn),
+          ('scope',w2lapp.searchform.SEARCH_SCOPE_STR_ONELEVEL),
+          ('filterstr',u'(objectClass=*)'),
+        ),
+        title=u'Descend into tree from here',
+      ),
+      form.applAnchor('searchform','Search',sid,
+        (('dn',dn),),
+        title=u'Enter search criteria in input form',
+      ),
     ))
 
     if dn or ls.l.get_option(ldap.OPT_PROTOCOL_VERSION)>=ldap.VERSION3:
@@ -551,16 +597,11 @@ def DataStr(sid,form,ls,dn,schema,attrtype_name,value,valueindex=0,commandbutton
 # Ausdrucken eines HTML-Kopfes mit Titelzeile
 def PrintHeader(sid,outf,form,TitleMsg,link_css):
 
-  additional_http_header={
-    'Pragma':'no-cache',
-    'Cache-Control':'no-store,no-cache,max-age=0,must-revalidate',
-    'X-XSS-Protection':'0',
-    'X-DNS-Prefetch-Control':'off',
-    'X-Content-Type-Options':'nosniff',
-    'X-FRAME-OPTIONS':'DENY',
-  }
+  additional_http_header = {}
+  additional_http_header.update(w2lapp.cnf.misc.http_headers)
 
-  if form.env.get('HTTPS','off')=='on':
+  if form.env.get('HTTPS','off')=='on' and \
+     not 'Strict-Transport-Security' in additional_http_header:
     additional_http_header['Strict-Transport-Security']='max-age=15768000 ; includeSubDomains'
 
   pyweblib.httphelper.SendHeader(
@@ -583,13 +624,54 @@ def PrintHeader(sid,outf,form,TitleMsg,link_css):
   return # PrintHeader()
 
 
+def URLRedirect(
+  outf,
+  form,
+  target_url,
+  refresh_time=0,
+  redirect_msg='Redirecting...',
+  link_text='Click here to manually follow redirect'
+):
+  """
+  Outputs HTML text with redirecting <head> section.
+  """
+
+  additional_http_header = {}
+  additional_http_header.update(w2lapp.cnf.misc.http_headers)
+
+  if form.env.get('HTTPS','off')=='on' and \
+     not 'Strict-Transport-Security' in additional_http_header:
+    additional_http_header['Strict-Transport-Security']='max-age=15768000 ; includeSubDomains'
+
+  pyweblib.httphelper.SendHeader(outf,additional_header=additional_http_header)
+
+  # Write out stub body with just a short redirect HTML snippet
+  outf.write("""
+<html>
+  <head>
+    <meta http-equiv="refresh" content="{refresh_time:d}; url={target_url}">
+  </head>
+  <body>
+    <p>{redirect_msg}</p>
+    <a href="{target_url}">{link_text}</a>
+  </body>
+</html>
+    """.format(
+      refresh_time = refresh_time,
+      target_url = target_url,
+      redirect_msg = redirect_msg,
+      link_text = link_text,
+    )
+  )
+  return # URLRedirect()
+
 # Ausdrucken eines HTML-Endes
 def PrintFooter(f,form):
   f.write('<a id="web2ldap_bottom"></a></body>\n</html>\n')
   f.flush()
 
 
-def SearchRootField(form,ls,dn,name='dn',text=u'Search Root',default=None):
+def SearchRootField(form,ls,dn,name='dn',text=u'Search Root',default=None,search_root_searchurl=None):
   """Prepare input field for search root"""
 
   def sortkey_func(s):
@@ -602,7 +684,28 @@ def SearchRootField(form,ls,dn,name='dn',text=u'Search Root',default=None):
   else:
     dn_select_list = []
   dn_select_list = msbase.union(ls.namingContexts,dn_select_list,ignorecase=1)
+  if search_root_searchurl:
+    slu = ldapurl.LDAPUrl(search_root_searchurl.encode(ls.charset))
+    try:
+      ldap_result = ls.l.search_s(
+        slu.dn,
+        slu.scope,
+        slu.filterstr,
+        attrlist=['1.1'],
+      )
+    except ldap.LDAPError:
+      pass
+    else:
+      dn_select_list = msbase.union(
+        [
+          ls.uc_decode(ldap_dn)[0]
+          for ldap_dn,ldap_enty in ldap_result
+        ],
+        dn_select_list,
+        ignorecase=1
+      )
   dn_select_list.append((u'',u'- World -'))
+  dn_select_list = list(set(dn_select_list))
   dn_select_list.sort(key=sortkey_func)
   srf = pyweblib.forms.Select(
     name,text,1,
