@@ -9,8 +9,6 @@ see http://www.web2ldap.de for details
 This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
-
-$Id: handler.py,v 1.312 2013/02/13 19:34:13 michael Exp $
 """
 
 NON_INTERACTIVE_LOGIN_MECHS = set(['EXTERNAL','GSSAPI'])
@@ -189,7 +187,7 @@ def HandleHTTPRequest(inf,outfile,errfile,env,script_name=None):
         # Explicitly remove old session
         session.deleteSession(sid)
         # Redirect to start page to avoid people bookmarking disconnect URL
-        pyweblib.httphelper.URLRedirect(outf,form.script_name,0,'Disconnect...')
+        w2lapp.gui.URLRedirect(outf,form,form.script_name,0,redirect_msg='Disconnecting...')
         return
       elif command=='':
         # New connect => remove old session if necessary
@@ -273,6 +271,13 @@ def HandleHTTPRequest(inf,outfile,errfile,env,script_name=None):
         # Extract the connection parameters from form fields
         form.getInputFields(ignoreEmptyFields=0)
 
+        try:
+          del_sid = form.field['delsid'].value[0]
+        except IndexError,e:
+          pass
+        else:
+          session.deleteSession(del_sid)
+
         if 'ldapurl' in form.inputFieldNames:
           # One form parameter with LDAP URL
           ldap_url_input = form.field['ldapurl'].value[0]
@@ -567,6 +572,33 @@ def HandleHTTPRequest(inf,outfile,errfile,env,script_name=None):
         who=who,relogin=1
       )
 
+    except ldapsession.PWD_EXPIRATION_WARNING,e:
+      # Setup what's required to the case command=='passwd'
+      ls.setDN(dn or e.who)
+      form.addField(pyweblib.forms.Select('passwd_scheme',u'Password hash scheme',1,options=w2lapp.passwd.available_hashtypes,default=w2lapp.passwd.available_hashtypes[-1]))
+      form.addField(pyweblib.forms.Checkbox('passwd_ntpasswordsync',u'Sync ntPassword for Samba',1,default="yes",checked=1))
+      form.addField(pyweblib.forms.Checkbox('passwd_settimesync',u'Sync password setting times',1,default="yes",checked=1))
+      # Directly generate the change password form
+      w2lapp.passwd.PasswdForm(
+        sid,outf,command,form,ls,dn,None,
+        None,e.who.decode(ls.charset),None,
+        'Password change needed',
+        form.utf2display(
+          u'Password will expire in %s!' % (
+            w2lapp.gui.ts2repr(
+              (
+                (u'weeks',604800),
+                (u'days',86400),
+                (u'hours',3600),
+                (u'mins',60),
+                (u'secs',1),
+              ),
+              u' ',
+              e.timeBeforeExpiration,
+            )
+        )),
+      )
+
     except ldapsession.PasswordPolicyException,e:
       # Setup what's required to the case command=='passwd'
       ls.setDN(dn or e.who)
