@@ -10,8 +10,6 @@ see http://www.web2ldap.de for details
 This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
-
-$Id: passwd.py,v 1.104 2012/12/12 21:11:24 michael Exp $
 """
 
 import time,string,binascii,random,hashlib,ldap,ldaputil.base,ldaputil.passwd, \
@@ -189,7 +187,7 @@ def PasswdForm(
       ]
       form.field['passwd_scheme'].options = default_hashtypes
 
-  nthash_available = all_attrs_dict.has_key('1.3.6.1.4.1.7165.2.1.2') or all_attrs_dict.has_key('1.3.6.1.4.1.7165.2.1.25')
+  nthash_available = all_attrs_dict.has_key('1.3.6.1.4.1.7165.2.1.25')
   show_clientside_pw_fields = passwd_action=='setuserpassword' and not unicode_pwd_avail
 
   passwd_template_str = w2lapp.gui.ReadTemplate(form,ls,'passwd_template',u'password form')
@@ -280,11 +278,21 @@ def w2l_Passwd(sid,outf,command,form,ls,dn,connLDAPUrl):
 
     password_attr_types_msg = ''
 
+    passwd_modlist = w2lapp.cnf.GetParam(ls,'passwd_modlist',[])
+
+    # Extend with appropriate user-must-change-password-after-reset attribute
+    if passwd_forcechange:
+      # draft-behera-password-policy
+      if sub_schema.sed[AttributeType].has_key('1.3.6.1.4.1.42.2.27.8.1.22'):
+        passwd_modlist.append((ldap.MOD_REPLACE,'pwdReset','TRUE'))
+      # MS AD
+      elif all_attrs_dict.has_key('1.2.840.113556.1.4.96'):
+        passwd_modlist.append((ldap.MOD_REPLACE,'pwdLastSet','0'))
+
     if not passwd_action:
 
       raise w2lapp.core.ErrorExit(u'No password action chosen.')
 
-
     elif passwd_action=='passwdextop':
 
       #-------------------------------------------------------
@@ -306,10 +314,8 @@ def w2l_Passwd(sid,outf,command,form,ls,dn,connLDAPUrl):
         )
         return
       else:
-        if passwd_forcechange:
-          # Check for draft-behera-password-policy
-          if sub_schema.sed[AttributeType].has_key('1.3.6.1.4.1.42.2.27.8.1.22'):
-            ls.modifyEntry(passwd_who,[(ldap.MOD_REPLACE,'pwdReset','TRUE')])
+        if passwd_modlist:
+          ls.modifyEntry(passwd_who,passwd_modlist)
         if no_passwd_input:
           password_attr_types_msg = 'Generated password set by the server: %s' % (form.utf2display(passwd_input))
         else:
@@ -321,8 +327,6 @@ def w2l_Passwd(sid,outf,command,form,ls,dn,connLDAPUrl):
       # Modify password via Modify Request
       #-------------------------------------------------------
 
-      modlist = []
-
       all_attrs_dict = GetAllAllowedAttributes(sub_schema,user_objectclasses)
 
       if not all_attrs_dict.has_key('2.5.4.35'):
@@ -333,7 +337,7 @@ def w2l_Passwd(sid,outf,command,form,ls,dn,connLDAPUrl):
             if user_objectclasses and \
                not userpassword_class.lower() in user_objectclasses and \
                sub_schema.get_inheritedattr(ldap.schema.ObjectClass,userpassword_class,'kind')==2:
-              modlist.append((ldap.MOD_ADD,'objectClass',userpassword_class))
+              passwd_modlist.append((ldap.MOD_ADD,'objectClass',userpassword_class))
               break
           except KeyError:
             pass
@@ -368,12 +372,12 @@ def w2l_Passwd(sid,outf,command,form,ls,dn,connLDAPUrl):
           old_passwd_value = userPassword.encodePassword(old_password,'')
 
       if OwnPwdChange(ls,passwd_who) and old_password:
-        modlist.extend((
+        passwd_modlist.extend((
           (ldap.MOD_DELETE,passwd_attr_type,[old_passwd_value]),
           (ldap.MOD_ADD,passwd_attr_type,[new_passwd_value]),
         ))
       else:
-        modlist.append(
+        passwd_modlist.append(
           (ldap.MOD_REPLACE,passwd_attr_type,[new_passwd_value]),
         )
 
@@ -382,39 +386,20 @@ def w2l_Passwd(sid,outf,command,form,ls,dn,connLDAPUrl):
       pwd_change_timestamp = time.time()
 
       if passwd_settimesync and all_attrs_dict.has_key('1.3.6.1.1.1.1.5'):
-        modlist.append((ldap.MOD_REPLACE,'shadowLastChange',str(int(pwd_change_timestamp/86400))))
-
-      if passwd_forcechange:
-        # draft-behera-password-policy
-        if sub_schema.sed[AttributeType].has_key('1.3.6.1.4.1.42.2.27.8.1.22'):
-          modlist.append((ldap.MOD_REPLACE,'pwdReset','TRUE'))
-        # MS AD
-        elif all_attrs_dict.has_key('1.2.840.113556.1.4.96'):
-          modlist.append((ldap.MOD_REPLACE,'pwdLastSet','0'))
+        passwd_modlist.append((ldap.MOD_REPLACE,'shadowLastChange',str(int(pwd_change_timestamp/86400))))
 
       passwd_ntpasswordsync = form.getInputValue('passwd_ntpasswordsync',['no'])[0]=='yes'
 
       # Samba password synchronization if requested
-      if passwd_ntpasswordsync:
-
-        if all_attrs_dict.has_key('1.3.6.1.4.1.7165.2.1.2'):
-          modlist.append((ldap.MOD_REPLACE,'ntPassword',NtPasswordHash(passwd_input)))
-
-        if all_attrs_dict.has_key('1.3.6.1.4.1.7165.2.1.25'):
-          modlist.append((ldap.MOD_REPLACE,'sambaNTPassword',NtPasswordHash(passwd_input)))
-
-      if passwd_settimesync and passwd_ntpasswordsync:
-
-        if all_attrs_dict.has_key('1.3.6.1.4.1.7165.2.1.3'):
-          modlist.append((ldap.MOD_REPLACE,'pwdLastSet',str(int(pwd_change_timestamp))))
-
-        if all_attrs_dict.has_key('1.3.6.1.4.1.7165.2.1.27'):
-          modlist.append((ldap.MOD_REPLACE,'sambaPwdLastSet',str(int(pwd_change_timestamp))))
+      if passwd_ntpasswordsync and all_attrs_dict.has_key('1.3.6.1.4.1.7165.2.1.25'):
+        passwd_modlist.append((ldap.MOD_REPLACE,'sambaNTPassword',NtPasswordHash(passwd_input)))
+        if passwd_settimesync and all_attrs_dict.has_key('1.3.6.1.4.1.7165.2.1.27'):
+          passwd_modlist.append((ldap.MOD_REPLACE,'sambaPwdLastSet',str(int(pwd_change_timestamp))))
 
       password_attr_types_msg = 'Password-related attributes set: %s' % (', '.join(
         [
-          '%s' % (attr_type)
-          for mod, attr_type, attr_values in modlist
+          form.utf2display(unicode(attr_type))
+          for mod, attr_type, attr_values in passwd_modlist
         ]
       ))
       if no_passwd_input:
@@ -422,7 +407,7 @@ def w2l_Passwd(sid,outf,command,form,ls,dn,connLDAPUrl):
 
       # Modify password
       try:
-        ls.modifyEntry(passwd_who,modlist)
+        ls.modifyEntry(passwd_who,passwd_modlist)
       except (ldap.CONSTRAINT_VIOLATION,ldap.UNWILLING_TO_PERFORM),e:
         PasswdForm(
           sid,outf,command,form,ls,dn,sub_schema,
