@@ -10,8 +10,6 @@ see http://www.web2ldap.de for details
 This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
-
-$Id: read.py,v 1.174 2013/01/02 17:47:47 michael Exp $
 """
 
 import pyweblib.forms,ldap.schema,ldaputil.schema,pyweblib.httphelper, \
@@ -48,7 +46,6 @@ def PrintTemplateOutput(sid,outf,form,ls,sub_schema,entry,display_entry,cnf_key,
         except IOError:
           outf.write('<p class="ErrorMessage">I/O error during reading template file for object class <var>%s</var>.</p>' % oc)
         else:
-          display_entry.sep = '<br>'
           try:
             template_attr_oid_set = set([
               sub_schema.getoid(ldap.schema.AttributeType,attr_type_name)
@@ -110,15 +107,17 @@ def generate_vcard(template_str,vcard_entry):
 
 class DisplayEntry(ldaputil.schema.Entry):
 
-  def __init__(self,sid,form,ls,dn,schema,entry):
-    ldaputil.schema.Entry.__init__(self,schema,None,entry)
-    self.entry = ldaputil.schema.Entry(schema,None,entry)
+  def __init__(self,sid,form,ls,dn,schema,entry,sep_attr,commandbutton):
+    assert type(dn)==type(u'')
+    ldaputil.schema.Entry.__init__(self,schema,dn.encode(ls.charset),entry)
+    self.entry = ldaputil.schema.Entry(schema,dn.encode(ls.charset),entry)
     self.sid = sid
     self.form = form
     self.ls = ls
     self.dn = dn
-    self.sep = None
     self.invalid_attrs = set()
+    self.sep_attr = sep_attr
+    self.commandbutton = commandbutton
 
   def __getitem__(self,nameoroid):
     oid = self._at2key(nameoroid)[0]
@@ -132,7 +131,7 @@ class DisplayEntry(ldaputil.schema.Entry):
       for i in range(len(values)):
         attr_instance = syntax_se(self.sid,self.form,self.ls,self.dn,self._s,nameoroid,values[i],self.entry)
         try:
-          attr_value_html = attr_instance.displayValue(valueindex=i,commandbutton=1)
+          attr_value_html = attr_instance.displayValue(valueindex=i,commandbutton=self.commandbutton)
         except UnicodeError:
           # Fall back to hex-dump output
           attr_instance = w2lapp.schema.syntaxes.OctetString(self.sid,self.form,self.ls,self.dn,self._s,nameoroid,values[i],self.entry)
@@ -143,10 +142,11 @@ class DisplayEntry(ldaputil.schema.Entry):
           attr_value_html = '<strike>%s</strike>' % (attr_value_html)
           self.invalid_attrs.add(nameoroid)
         result.append(attr_value_html)
-      if self.sep is None:
-        return result
+      if self.sep_attr!=None:
+        value_sep = getattr(attr_instance,self.sep_attr)
+        return value_sep.join(result)
       else:
-        return self.sep.join(result)
+        return result
 
 
 def getOperationAttrsTemplate(ls,accept_language):
@@ -304,12 +304,13 @@ def w2l_Read(
       if search_result:
         entry.update(search_result[0][1])
 
-  display_entry = DisplayEntry(sid,form,ls,dn,sub_schema,entry)
+  display_entry = DisplayEntry(sid,form,ls,dn,sub_schema,entry,'readSep',1)
 
   # Save session into database mainly for storing LDAPSession cache
   session.storeSession(sid,ls)
 
-  if len(wanted_attrs)==1:
+  if len(wanted_attrs)==1 and \
+     not wanted_attrs[0] in ('*','+'):
 
     # Display a single binary attribute either with a registered
     # viewer or just by sending the data blob with appropriate MIME-type
@@ -445,10 +446,12 @@ def w2l_Read(
           else:
             nomatching_attrs.append(a)
 
+    display_entry.sep_attr = None
     PrintAttrList(sid,outf,ls,form,dn,sub_schema,display_entry,required_attrs,'Required Attributes')
     PrintAttrList(sid,outf,ls,form,dn,sub_schema,display_entry,allowed_attrs,'Allowed Attributes')
     PrintAttrList(sid,outf,ls,form,dn,sub_schema,display_entry,collective_attrs,'Collective Attributes')
     PrintAttrList(sid,outf,ls,form,dn,sub_schema,display_entry,nomatching_attrs,'Various Attributes')
+    display_entry.sep_attr = 'readSep'
 
     # Display operational attributes with template as footer
     if read_output==u'template':
