@@ -1,11 +1,9 @@
 # -*- coding: utf-8 -*-
 """
 web2ldap plugin classes for Active Directory (for some information see draft-armijo-ldap-syntax)
-
-$Id: activedirectory.py,v 1.48 2012/06/01 20:48:26 michael Exp $
 """
 
-import time,utctime,struct,pyweblib,ldap.cidict,w2lapp.cnf,mspki.util
+import time,utctime,struct,uuid,pyweblib,ldap.cidict,w2lapp.cnf,w2lapp.searchform,mspki.util
 
 from w2lapp.gui import DisplayDN
 
@@ -196,11 +194,10 @@ class OtherSID(ObjectSID):
             'searchform','&raquo;',self._sid,
             [
               ('dn',self._dn),
-              ('filterstr', '(objectSid=%s)' % (
-                  self._form.utf2display(sddl_str)
-                )
-              ),
-              ('searchform_mode','exp'),
+              ('searchform_mode',u'adv'),
+              ('search_attr',u'objectSid'),
+              ('search_option',w2lapp.searchform.SEARCH_OPT_IS_EQUAL),
+              ('search_string',sddl_str),
             ],
             title=u'Search by SID',
         )
@@ -599,6 +596,35 @@ class ObjectSecurityDescriptor(OctetString):
   desc = 'Object-Security-Descriptor'
 
 
+class MsAdGUID(OctetString):
+  oid = 'MsAdGUID-oid'
+  desc = 'GUID in Active Directory'
+
+  def sanitizeInput(self,attrValue):
+    try:
+      object_guid_uuid = uuid.UUID(attrValue.replace(':',''))
+    except ValueError:
+      return OctetString.sanitizeInput(self,attrValue)
+    else:
+      return object_guid_uuid.bytes
+
+  def displayValue(self,valueindex=0,commandbutton=0):
+    object_guid_uuid = uuid.UUID(bytes=self.attrValue)
+    return '{%s}<br>%s' % (
+      str(object_guid_uuid),
+      OctetString.displayValue(self,valueindex=0,commandbutton=0),
+    )
+
+syntax_registry.registerAttrType(
+  MsAdGUID.oid,[
+    '1.2.840.113556.1.4.2',    # objectGUID
+    '1.2.840.113556.1.4.1224', # parentGUID
+    '1.2.840.113556.1.4.340',  # rightsGuid
+    '1.2.840.113556.1.4.362',  # siteGUID
+  ]
+)
+
+
 class Interval(MicrosoftLargeInteger):
   oid = 'Interval-oid'
   desc = 'Large integer with timestamp expressed as 100 nanoseconds since 1601-01-01 00:00'
@@ -758,6 +784,23 @@ syntax_registry.registerAttrType(
 )
 
 
+class MsDSSupportedEncryptionTypes(BitArrayInteger):
+  oid = 'MsDSSupportedEncryptionTypes-oid'
+  flag_desc_table = (
+    (u'KERB_ENCTYPE_DES_CBC_CRC',0x00000001),
+    (u'KERB_ENCTYPE_DES_CBC_MD5',0x00000002),
+    (u'KERB_ENCTYPE_RC4_HMAC_MD5',0x00000004),
+    (u'KERB_ENCTYPE_AES128_CTS_HMAC_SHA1_96',0x00000008),
+    (u'KERB_ENCTYPE_AES256_CTS_HMAC_SHA1_96',0x00000010),
+  )
+  
+syntax_registry.registerAttrType(
+  MsDSSupportedEncryptionTypes.oid,[
+    '1.2.840.113556.1.4.1963', # msDS-SupportedEncryptionTypes
+  ]
+)
+
+
 # Register all syntax classes in this module
 for symbol_name in dir():
   syntax_registry.registerSyntaxClass(eval(symbol_name))
