@@ -1,11 +1,9 @@
 # -*- coding: utf-8 -*-
 """
 web2ldap plugin classes for schema elements defined in RFC2307
-
-$Id: nis.py,v 1.26 2013/02/08 18:50:11 michael Exp $
 """
 
-import re,pyweblib.forms,ldap.schema,ldaputil.schema
+import re,pyweblib.forms,ldap.schema,ldaputil.schema,w2lapp.searchform
 
 from w2lapp.schema.syntaxes import SelectList,IA5String,IntegerRange, \
                          IPHostAddress,IPNetworkAddress,IPServicePortNumber,MacAddress, \
@@ -50,20 +48,23 @@ class GidNumber(DynamicValueSelectList,IntegerRange):
         return r[0]
       if 'posixGroup' in ocs:
         title = u'Search group members'
-        filterstr_template = '(&(|(objectClass=posixAccount)(objectClass=shadowAccount))(gidNumber=%s))'
+        searchform_params = [
+          ('dn',self._dn),
+          ('searchform_mode',u'adv'),
+          ('search_attr',u'objectClass'),
+          ('search_option',w2lapp.searchform.SEARCH_OPT_IS_EQUAL),
+          ('search_string',u'posixGroup'),
+          ('search_attr',u'gidNumber'),
+          ('search_option',w2lapp.searchform.SEARCH_OPT_IS_EQUAL),
+          ('search_string',self._ls.uc_decode(self.attrValue)[0]),
+        ]
       else:
         title = None
-        filterstr_template = None
-      if title and filterstr_template:
+        searchform_params = None
+      if title and searchform_params:
         r.append(self._form.applAnchor(
             'searchform','&raquo;',self._sid,
-            [
-              ('dn',self._dn),
-              ('filterstr', filterstr_template % (
-                escape_filter_chars(self._ls.uc_decode(self.attrValue)[0])
-              )),
-              ('searchform_mode','exp'),
-            ],
+            searchform_params,
             title=title,
         ))
       return ' '.join(r)
