@@ -10,11 +10,9 @@ see http://www.web2ldap.de for details
 This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
-
-$Id: syntaxes.py,v 1.285 2013/02/16 16:38:07 michael Exp $
 """
 
-import re,imghdr,sndhdr,urllib,datetime,time,utctime,ldap,ldapurl,pyweblib.forms, \
+import re,imghdr,sndhdr,urllib,uuid,datetime,time,utctime,ldap,ldapurl,pyweblib.forms, \
        ldaputil.base,ldap.schema,xml.etree.ElementTree, \
        w2lapp.viewer,w2lapp.form,w2lapp.gui,mspki.util,w2lapp.cnf,ldaputil.schema
 
@@ -22,8 +20,6 @@ from types import StringType,UnicodeType,IntType,LongType,FloatType,ClassType
 
 from ldaputil.base import is_dn
 
-from ldap.dn import explode_dn
-
 try:
   from netaddr import IPAddress,IPNetwork
 except ImportError:
@@ -112,6 +108,9 @@ class LDAPSyntax:
   fileExt = 'bin'
   editable = 1
   reObj = None
+  searchSep = '<br>'
+  readSep = '<br>'
+  fieldSep = '<br>'
 
   def __init__(self,sid,form,ls,dn,schema,attrType,attrValue,entry=None):
     self.attrType = attrType
@@ -248,13 +247,6 @@ class Audio(Binary):
       )
 
 
-class CertificatePair(Binary):
-  oid = '1.3.6.1.4.1.1466.115.121.1.10'
-  desc = 'X.509 Certificate Pair'
-  mimeType = 'application/pkix-cert'
-  fileExt = 'cer'
-
-
 class AttributeCertificate(Binary):
   oid = '1.3.6.1.4.1.4203.666.11.10.2.1'
   desc = 'X.509 Attribute Certificate'
@@ -262,130 +254,6 @@ class AttributeCertificate(Binary):
   fileExt = 'cer'
 
 
-class CertificateSimpleClass(Binary):
-  oid = '1.3.6.1.4.1.1466.115.121.1.8'
-  desc = 'X.509 Certificate'
-  mimeType = 'application/pkix-cert'
-  fileExt = 'cer'
-  pemBeginText = '-----BEGIN CERTIFICATE-----'
-  pemEndText = '-----END CERTIFICATE-----'
-
-  def sanitizeInput(self,attrValue):
-    pem_begin_pos = attrValue.find(self.pemBeginText)
-    if pem_begin_pos>=0:
-      pem_end_pos = attrValue.find(self.pemEndText,pem_begin_pos+len(self.pemBeginText))
-      if pem_end_pos>=0:
-        return attrValue[pem_begin_pos+len(self.pemBeginText):pem_end_pos].strip().decode('base64')
-      else:
-        return attrValue
-    else:
-      return attrValue
-
-  def getMimeType(self):
-    if self._form.browser_type in ['Mozilla','Opera']:
-      return 'application/x-x509-email-cert'
-    return self.mimeType
-
-  def displayValue(self,valueindex=0,commandbutton=0):
-    return '%d bytes | %s' % (
-      len(self.attrValue),
-      self._form.applAnchor(
-        'read','View/Load',self._sid,
-        [
-          ('dn',self._dn),
-          ('read_attr',self.attrType),
-          ('read_attrindex',str(valueindex)),
-          ('read_attrmode','view'),
-        ]
-      )
-    )
-
-
-try:
-  import M2Crypto
-except ImportError:
-  Certificate = CertificateSimpleClass
-else:
-
-  class CertificateM2Class(CertificateSimpleClass):
-    cert_display_template = """
-      <dl>
-        <dt>Issuer:</dt>
-        <dd>{cert_issuer_dn}</dd>
-        <dt>Subject</dt>
-        <dd>{cert_subject_dn}</dd>
-        <dt>Serial No.</dt>
-        <dd>{cert_serial_number_dec} ({cert_serial_number_hex})</dd>
-        <dt>Validity period</dt>
-        <dd>from {cert_not_before} until {cert_not_after}</dd>
-      </dl>
-      """
-
-    def displayValue(self,valueindex=0,commandbutton=0):
-      links_html = CertificateSimpleClass.displayValue(self,valueindex,commandbutton)
-      try:
-        x509 = M2Crypto.X509.load_cert_string(self.attrValue,M2Crypto.X509.FORMAT_DER)
-      except:
-        cert_html = ''
-        raise
-      else:
-        cert_issuer_dn = ','.join(
-          explode_dn(x509.get_issuer().as_text(flags=M2Crypto.m2.XN_FLAG_RFC2253))
-        ).decode('utf-8')
-        cert_subject_dn = ','.join(
-          explode_dn(x509.get_subject().as_text(flags=M2Crypto.m2.XN_FLAG_RFC2253))
-        ).decode('utf-8')
-        cert_serial_number = int(x509.get_serial_number())
-        try:
-          cert_not_before = x509.get_not_before().get_datetime()
-        except (ValueError,NameError):
-          cert_not_before = 'ValueError'
-        else:
-          cert_not_before = cert_not_before.strftime('%Y-%m-%dT%H-%M-%S %Z')
-        try:
-          cert_not_after = x509.get_not_after().get_datetime()
-        except (ValueError,NameError):
-          cert_not_after = 'ValueError'
-        else:
-          cert_not_after = cert_not_after.strftime('%Y-%m-%dT%H-%M-%S %Z')
-        cert_html = self.cert_display_template.format(
-          cert_issuer_dn = self._form.utf2display(cert_issuer_dn),
-          cert_subject_dn = self._form.utf2display(cert_subject_dn),
-          cert_serial_number_dec = str(cert_serial_number),
-          cert_serial_number_hex = hex(cert_serial_number),
-          cert_not_before = cert_not_before,
-          cert_not_after = cert_not_after,
-        )
-      return ''.join((cert_html,links_html))
-
-  Certificate = CertificateM2Class
-
-
-class CACertificate(Certificate):
-  oid = 'CACertificate-oid'
-  desc = 'X.509 CA Certificate'
-  mimeType = 'application/x-x509-ca-cert'
-  pemBeginText = '-----BEGIN TRUSTED CERTIFICATE-----'
-  pemEndText = '-----END TRUSTED CERTIFICATE-----'
-
-  def getMimeType(self):
-    return self.mimeType
-
-
-class CertificateRevocationList(CertificateSimpleClass):
-  oid = '1.3.6.1.4.1.1466.115.121.1.9'
-  desc = 'Certificate Revocation List'
-  mimeType = 'application/pkix-crl'
-  fileExt = 'crl'
-  pemBeginText = '-----BEGIN X509 CRL-----'
-  pemEndText = '-----END X509 CRL-----'
-
-  def getMimeType(self):
-    if self._form.browser_type in ['Mozilla','Opera']:
-      return 'application/x-pkcs7-crl'
-    return self.mimeType
-
-
 class DirectoryString(LDAPSyntax):
   oid = '1.3.6.1.4.1.1466.115.121.1.15'
   desc = 'Directory String'
@@ -735,7 +603,7 @@ class Image(Binary):
   imageFormat = None
 
   def _validate(self,attrValue):
-    return imghdr.test_jpeg(attrValue,None)==self.imageFormat.lower()
+    return imghdr.what(None,attrValue)==self.imageFormat.lower()
 
   def sanitizeInput(self,attrValue):
     if not self._validate(attrValue) and PILImage:
@@ -1110,6 +978,48 @@ class DaysSinceEpoch(Integer):
       return Integer.displayValue(self,valueindex,commandbutton)
 
 
+class Timespan(Integer):
+  oid = 'Timespan-oid'
+  desc = 'Time span in seconds'
+  time_divisors = (
+    (u'weeks',604800),
+    (u'days',86400),
+    (u'hours',3600),
+    (u'mins',60),
+    (u'secs',1),
+  )
+  sep = u','
+
+  def sanitizeInput(self,attrValue):
+    if attrValue:
+      try:
+        result = str(w2lapp.gui.repr2ts(self.time_divisors,self.sep,attrValue))
+      except ValueError:
+        result = Integer.sanitizeInput(self,attrValue)
+    else:
+      result = attrValue
+    return result
+
+  def formValue(self):
+    if not self.attrValue:
+      return self.attrValue
+    try:
+      result = w2lapp.gui.ts2repr(self.time_divisors,self.sep,int(self.attrValue))
+    except ValueError:
+      result = Integer.formValue(self)
+    return result
+
+  def displayValue(self,valueindex=0,commandbutton=0):
+    try:
+      result = self._form.utf2display('%s (%s)' % (
+        w2lapp.gui.ts2repr(self.time_divisors,self.sep,int(self.attrValue)),
+        Integer.displayValue(self,valueindex,commandbutton)
+      ))
+    except ValueError:
+      result = Integer.displayValue(self,valueindex,commandbutton)
+    return result
+
+
 class SelectList(DirectoryString):
   """
   Base class for dictionary based select lists which
@@ -1367,7 +1277,7 @@ class DynamicDNSelectList(DynamicValueSelectList,DistinguishedName):
   def _readReferencedEntry(self,dn):
     try:
       ldap_result = self._ls.readEntry(
-        dn,
+        dn.decode(self._ls.charset),
         attrtype_list=self.lu_obj.attrs,
         search_filter=self.lu_obj.filterstr or '(objectClass=*)',
       )
@@ -1522,11 +1432,22 @@ class BitArrayInteger(MultilineText,Integer):
     )
 
 
+class GSER(DirectoryString):
+  oid = 'GSER-oid'
+  desc = 'GSER syntax (see RFC 3641)'
+
+
 class UUID(IA5String):
   oid = '1.3.6.1.1.16.1'
   desc = 'UUID'
   reObj = re.compile('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$')
 
+  def sanitizeInput(self,attrValue):
+    try:
+      return str(uuid.UUID(attrValue.replace(':','')))
+    except ValueError:
+      return attrValue
+
 
 class DNSDomain(IA5String):
   oid = 'DNSDomain-oid'
@@ -1535,7 +1456,7 @@ class DNSDomain(IA5String):
 
   def sanitizeInput(self,attrValue):
     return '.'.join([
-      dc.encode('idna')
+      dc.lower().encode('idna')
       for dc in attrValue.decode(self._form.accept_charset).split(u'.')
     ])
 
@@ -1626,9 +1547,11 @@ class XmlValue(MultilineText):
 try:
   # Try to import optional module pisces
   from pisces import asn1
+
 except ImportError:
-  # Well, silently ignore and don't do anything
-  pass
+  # Fall-back class is Binary
+  ASN1Object = Binary
+
 else:
 
   class ASN1Object(Binary):
@@ -1655,7 +1578,7 @@ else:
         pisces_oid = asn1.OID(tuple(map(int,attrValue.split('.'))))
         desc = mspki.asn1helper.GetOIDDescription(
           pisces_oid,
-          w2lapp.viewer.oids,
+          mspki.asn1helper.oids,
           includeoid=1
         )
       except ValueError:
