@@ -10,8 +10,6 @@ see http://www.web2ldap.de for details
 This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
-
-$Id: viewer.py,v 1.112 2012/12/31 17:04:30 michael Exp $
 """
 
 import types,ldap,msbase,w2lapp.gui
@@ -162,7 +160,7 @@ class DisplaySchemaElement:
           ])
         else:
           value_output = ', '.join(displayNameOrOIDList(sid,form,dn,self.s,class_attr_value_list,se_class))
-        outf_lines.append('<dt>%s</dt>\n<dd><p>\n%s\n</p></dd>\n' % (text,value_output))
+        outf_lines.append('<dt>%s</dt>\n<dd>\n%s\n</dd>\n' % (text,value_output))
     return outf_lines # displayDetails()
 
   def display(self,sid,outf,form,ls,dn):
@@ -384,6 +382,20 @@ class DisplayAttributeType(DisplaySchemaElement):
           ', '.join(displayNameOrOIDList(sid,form,dn,self.s,mr_applicable_for,MatchingRule))
         ))
 
+    ###################################################################
+    # Display DIT content rules which reference attributes of this type
+    ###################################################################
+    attr_type_ref_list = []
+    for oc_oid,object_class_se in self.s.sed[ObjectClass].items():
+      object_class_se = self.s.get_obj(ObjectClass,oc_oid)
+      for dcr_at in object_class_se.must+object_class_se.may:
+        if dcr_at==at_oid or dcr_at in self.sei.names:
+          attr_type_ref_list.append(oc_oid)
+    if attr_type_ref_list:
+      outf_lines.append('<dt>Directly referencing object classes:</dt>\n<dd>\n%s\n</dd>\n' % (
+        ', '.join(displayNameOrOIDList(sid,form,dn,self.s,attr_type_ref_list,ObjectClass))
+      ))
+
     ###################################################################
     # Display object classes which may contain attributes of this type
     ###################################################################
@@ -394,10 +406,24 @@ class DisplayAttributeType(DisplaySchemaElement):
       if must.has_key(at_oid) or may.has_key(at_oid):
         attr_type_ref_list.append(oc_oid)
     if attr_type_ref_list:
-      outf_lines.append('<dt>Referencing object classes:</dt>\n<dd>\n%s\n</dd>\n' % (
+      outf_lines.append('<dt>Usable in these object classes:</dt>\n<dd>\n%s\n</dd>\n' % (
         ', '.join(displayNameOrOIDList(sid,form,dn,self.s,attr_type_ref_list,ObjectClass))
       ))
 
+    ###################################################################
+    # Display DIT content rules which reference attributes of this type
+    ###################################################################
+    attr_type_ref_list = []
+    for dcr_oid,dit_content_rule_se in self.s.sed[DITContentRule].items():
+      dit_content_rule_se = self.s.get_obj(DITContentRule,dcr_oid)
+      for dcr_at in dit_content_rule_se.must+dit_content_rule_se.may+dit_content_rule_se.nots:
+        if dcr_at==at_oid or dcr_at in self.sei.names:
+          attr_type_ref_list.append(dcr_oid)
+    if attr_type_ref_list:
+      outf_lines.append('<dt>Referencing DIT content rules:</dt>\n<dd>\n%s\n</dd>\n' % (
+        ', '.join(displayNameOrOIDList(sid,form,dn,self.s,attr_type_ref_list,DITContentRule))
+      ))
+
     ########################################################################
     # Display name forms which uses this attribute type for naming an entry
     ########################################################################
@@ -642,20 +668,22 @@ SCHEMA_VIEWER_CLASS = {
 
 
 def OIDInputForm(form,sid,dn,oid=None):
+  oid_input_field_html = w2lapp.form.OIDInput(
+    'oid',
+    u'OID or descriptive name of schema element',
+    default=oid
+  ).inputHTML(oid)
+  oid_class_select_html = form.field['oid_class'].inputHTML('')
   return form.formHTML(
     'oid','Search',sid,'GET',
     [('dn',dn)],
-    extrastr=w2lapp.form.OIDInput(
-      'oid',
-      u'OID or descriptive name of schema element',
-      default=oid
-    ).inputHTML(oid)
+    extrastr='\n'.join((oid_input_field_html,oid_class_select_html)),
   )
 
 
 def DisplayAllSchemaelements(sid,outf,form,ls,dn,schema,se_classes,se_list):
   se_list = se_list or []
-  se_classes = filter(None,se_classes or [])
+  se_classes = filter(None,se_classes or []) or SCHEMA_CLASS_MAPPING.values()
 
   w2lapp.gui.TopSection(
     sid,outf,form,ls,dn,'Schema elements',
@@ -690,9 +718,11 @@ def DisplayAllSchemaelements(sid,outf,form,ls,dn,schema,se_classes,se_list):
       schema_elements = oid_dict[schema_class]
       if not schema_elements:
         continue
-      outf.write('<h2>%s</h2>\n' % (SCHEMA_VIEWER_CLASS[schema_class].type_desc))
-      outf.write('<p>found %d</p>\n' % (len(schema_elements)))
-      outf.write(', '.join(displayNameOrOIDList(sid,form,dn,schema,schema_elements,schema_class)))
+      outf.write('<h2>%s</h2>\n<p>found %d</p>\n%s\n' % (
+        SCHEMA_VIEWER_CLASS[schema_class].type_desc,
+        len(schema_elements),
+        ',\n '.join(displayNameOrOIDList(sid,form,dn,schema,schema_elements,schema_class)),
+      ))
   else:
     outf.write("""<p>Hints:</p>
     <ul>
@@ -720,11 +750,15 @@ def w2l_DisplaySchemaElement(sid,outf,command,form,ls,dn):
 
   # Get input parameter from form input
   oid = form.getInputValue('oid',[None])[0]
-  se_class = SCHEMA_CLASS_MAPPING.get(form.getInputValue('oid_class',[''])[0].strip(),None)
+  se_classes = [
+    SCHEMA_CLASS_MAPPING[se_name.strip()]
+    for se_name in form.getInputValue('oid_class',[])
+    if se_name
+  ]
 
-  if oid is None:
+  if not oid or oid=='*':
     # Display entry page of schema browser
-    DisplayAllSchemaelements(sid,outf,form,ls,dn,sub_schema,[se_class],None)
+    DisplayAllSchemaelements(sid,outf,form,ls,dn,sub_schema,se_classes,None)
     return
 
   else:
@@ -747,21 +781,21 @@ def w2l_DisplaySchemaElement(sid,outf,command,form,ls,dn):
     else:
       cmp_method = None
 
-    if se_class and cmp_method is None:
+    if len(se_classes)==1 and cmp_method is None:
       # Display a single schema element referenced by OID and class
-      se_list = [sub_schema.get_obj(se_class,oid,None)]
+      se_list = [sub_schema.get_obj(se_classes[0],oid,None)]
     else:
       # Search schema element by OID
       se_list = []
       if cmp_method is None:
         # No wildcard search => just try to look up directly via name or OID
-        for schema_element_type in SCHEMA_VIEWER_CLASS.keys():
+        for schema_element_type in se_classes or SCHEMA_VIEWER_CLASS.keys():
           se = sub_schema.get_obj(schema_element_type,oid,None)
           if not se is None:
             se_list.append(se)
       else:
         # Do a wildcard search
-        for schema_element_type in SCHEMA_VIEWER_CLASS.keys():
+        for schema_element_type in se_classes or SCHEMA_VIEWER_CLASS.keys():
           for se in sub_schema.sed[schema_element_type].values():
             try:
               se_id = se.oid
