@@ -10,26 +10,29 @@ see http://www.web2ldap.de for details
 This module is distributed under the terms of the
 GPL (GNU GENERAL PUBLIC LICENSE) Version 2
 (see http://www.gnu.org/copyleft/gpl.html)
-
-$Id: search.py,v 1.217 2012/10/09 15:10:18 michael Exp $
 """
 
 import time,csv,urllib,ldap,ldap.schema.models,ldap.async,dsml, \
-       msbase,pyweblib.forms,pyweblib.httphelper,ldaputil.base, \
+       msbase,pyweblib.forms,pyweblib.httphelper,ldaputil.base,ldaputil.schema, \
        w2lapp.core,w2lapp.cnf,w2lapp.gui,w2lapp.searchform
 
 
 from ldapurl import LDAPUrl
+from ldaputil.base import escape_ldap_filter_chars
 from msbase import GrabKeys
 from w2lapp.schema.syntaxes import syntax_registry
 from ldaputil.controls import SearchNoOpControl
+from types import StringType,UnicodeType
+from ldapsession import LDAPLimitErrors
+
+SEARCH_NOOP_TIMEOUT = 5.0
 
 SizeLimitMsg = """
 <p class="ErrorMessage">
   <strong>
     Only partial results received. Try to refine search.
   </strong><br>
-  %s
+  {error_msg}
 </p>
 """
 
@@ -39,6 +42,7 @@ is_search_reference = {
   ldap.RES_SEARCH_REFERENCE:None
 }
 
+
 class excel_semicolon(csv.excel):
   """Describe the usual properties of Excel-generated TAB-delimited files."""
   delimiter = ';'
@@ -65,7 +69,7 @@ class CSVWriter(ldap.async.AsyncSearchHandler):
 
   def _processSingleResult(self,resultType,resultItem):
     if self._entryResultTypes.has_key(resultType):
-      entry = ldap.schema.models.Entry(self._s,resultItem[0],resultItem[1])
+      entry = ldaputil.schema.Entry(self._s,resultItem[0],resultItem[1])
       csv_row_list = []
       for attr_type in self._attr_types:
         csv_col_value_list = []
@@ -112,7 +116,7 @@ else:
 
     def _processSingleResult(self,resultType,resultItem):
       if self._entryResultTypes.has_key(resultType):
-        entry = ldap.schema.models.Entry(self._s,resultItem[0],resultItem[1])
+        entry = ldaputil.schema.Entry(self._s,resultItem[0],resultItem[1])
         csv_row_list = []
         for attr_type in self._attr_types:
           csv_col_value_list = []
@@ -166,76 +170,78 @@ def w2l_Search(sid,outf,command,form,ls,dn,connLDAPUrl):
       sid,outf,command,form,ls,dn,
       searchform_mode='adv',
       Msg='',
-      filterstr='',
-      scope=ldap.SCOPE_SUBTREE
+      filterstr=u'',
+      scope=scope
     )
     return
 
   # This should speed up things
   utf2display = form.utf2display
 
-  # Hmm, this retrieves sub schema sub entry for the search root.
-  # Theoretically it could be different for all search results.
-  # But what the hey...
-  sub_schema = ls.retrieveSubSchema(dn,w2lapp.cnf.GetParam(ls,'_schema',None))
-
   search_output = form.getInputValue('search_output',['table'])[0]
   search_opattrs = form.getInputValue('search_opattrs',['no'])[0]=='yes'
   search_root = form.getInputValue('search_root',[dn])[0]
 
+  # Hmm, this retrieves sub schema sub entry for the search root.
+  # Theoretically it could be different for all search results.
+  # But what the hey...
+  sub_schema = ls.retrieveSubSchema(search_root,w2lapp.cnf.GetParam(ls,'_schema',None))
+
   if scope==None:
     scope = ldap.SCOPE_SUBTREE
 
+  search_filter = form.getInputValue('filterstr',[filterstr])
+
   if ('search_option' in form.inputFieldNames) and \
      ('search_attr' in form.inputFieldNames) and \
      ('search_string' in form.inputFieldNames):
     search_mode = form.getInputValue('search_mode',[ur'(&%s)'])[0]
     search_option = form.getInputValue('search_option',[])
     search_attr = form.getInputValue('search_attr',[])
+    search_mr = form.getInputValue('search_mr',[None]*len(search_attr))
     search_string = form.getInputValue('search_string',[])
-    if (len(search_option)==len(search_attr)==len(search_string)):
+    if (len(search_option)==len(search_attr)==len(search_mr)==len(search_string)):
       # Get search mode
       # Build LDAP search filter from input data of advanced search form
-      search_filter = []
-      for i in range(len(form.field['search_option'].value)):
-        search_string,search_option,search_attr = \
-          form.field['search_string'].value[i], \
-          form.field['search_option'].value[i], \
-          form.field['search_attr'].value[i]
-        if search_option==r'(%s=*)':
-          search_filter.append(search_option % (search_attr))
-        elif search_string:
-          for attr_type in search_attr.split(','):
-            if not '*' in search_option:
-              # If an exact assertion value is needed we can normalize via plugin class
-              attr_instance = syntax_registry.attrInstance(None,form,ls,dn,sub_schema,attr_type,None,entry=None)
-              search_string = attr_instance.sanitizeInput(search_string)
-            search_filter.append(search_option % (
-              attr_type,ldap.filter.escape_filter_chars(search_string)
-            ))
-      if len(search_filter)==1:
-        filterstr = u''.join(search_filter)
-      elif len(search_filter)>1:
-        filterstr = search_mode % (u''.join(search_filter))
-      else:
-        w2lapp.searchform.w2l_SearchForm(
-          sid,outf,command,form,ls,dn,
-          searchform_mode='adv',
-          Msg='Empty search form data.',
-          filterstr='',
-          scope=ldap.SCOPE_SUBTREE
-        )
-        return
+      for i in range(len(search_attr)):
+        if search_option[i]==w2lapp.searchform.SEARCH_OPT_EXISTS:
+          search_filter.append(search_option.format(at=search_attr[i]))
+        elif search_string[i]:
+          if not '*' in search_option[i]:
+            # If an exact assertion value is needed we can normalize via plugin class
+            attr_instance = syntax_registry.attrInstance(
+              None,form,ls,dn,sub_schema,search_attr[i],None,entry=None
+            )
+            search_string[i] = attr_instance.sanitizeInput(search_string[i])
+          if search_mr[i]:
+            search_mr_string = ':%s:' % (search_mr[i])
+          else:
+            search_mr_string = ''
+          search_filter.append(search_option[i] % (
+            ''.join((search_attr[i],search_mr_string)),
+            escape_ldap_filter_chars(search_string[i])
+          ))
     else:
       raise w2lapp.core.ErrorExit(u'Invalid search form data.')
-  else:
-    filterstr = form.getInputValue(
-      'filterstr',[filterstr or u'(objectClass=*)']
-    )[0]
 
-  search_resminindex = int(
-    form.getInputValue('search_resminindex',['0'])[0]
-  )
+  # Wipe out all nullable search_filter list items
+  search_filter = filter(None,search_filter)
+
+  if not search_filter:
+    w2lapp.searchform.w2l_SearchForm(
+      sid,outf,command,form,ls,dn,
+      searchform_mode={True:'adv',False:'exp'}['search_attr' in form.inputFieldNames],
+      Msg='Empty search values.',
+      filterstr=u'',
+      scope=scope
+    )
+    return
+  elif len(search_filter)==1:
+    filterstr = search_filter[0]
+  elif len(search_filter)>1:
+    filterstr = search_mode % (u''.join(search_filter))
+
+  search_resminindex = int(form.getInputValue('search_resminindex',['0'])[0])
   search_resnumber = int(
     form.getInputValue(
       'search_resnumber',
@@ -268,6 +274,9 @@ def w2l_Search(sid,outf,command,form,ls,dn,connLDAPUrl):
     if a.strip()
   ]
 
+  search_attr_set = ldaputil.schema.SchemaElementOIDSet(sub_schema,ldap.schema.models.AttributeType,search_attrs)
+  search_attrs = search_attr_set.names()
+
   if search_output=='print':
     print_template_filenames_dict = w2lapp.cnf.GetParam(ls,'print_template',None)
     print_cols = w2lapp.cnf.GetParam(ls,'print_cols','4')
@@ -367,7 +376,10 @@ def w2l_Search(sid,outf,command,form,ls,dn,connLDAPUrl):
     # Give the user a chance to edit his bad search filter
     w2lapp.searchform.w2l_SearchForm(
       sid,outf,command,form,ls,dn,searchform_mode='exp',
-      Msg=w2lapp.gui.LDAPError2ErrMsg(e,form,charset=ls.charset),
+      Msg=' '.join((
+        w2lapp.gui.LDAPError2ErrMsg(e,form,charset=ls.charset),
+        form.utf2display(filterstr2),
+      )),
       filterstr=filterstr,
       scope=scope
     )
@@ -393,21 +405,15 @@ def w2l_Search(sid,outf,command,form,ls,dn,connLDAPUrl):
       # Retrieve the overall number of search results by resending the
       # search request without size limit but with the SearchNoOpControl attached
       if SearchNoOpControl.controlType in ls.rootDSE.get('supportedControl',[]):
-        msg_id = ls.l.search_ext(
-          search_root.encode(ls.charset),
-          scope,
-          filterstr=filterstr2.encode(ls.charset),
-          attrlist=['1.1'],
-          serverctrls=[SearchNoOpControl()],
-        )
-        _,_,_,search_response_ctrls = ls.l.result3(msg_id,all=1)
-        noop_srch_ctrl = [
-          c
-          for c in search_response_ctrls
-          if c.controlType==SearchNoOpControl.controlType
-        ]
-        if noop_srch_ctrl:
-          _,num_all_search_results,num_all_search_continuations = noop_srch_ctrl[0].resultCode,noop_srch_ctrl[0].numSearchResults,noop_srch_ctrl[0].numSearchContinuations
+        try:
+          num_all_search_results,num_all_search_continuations = ls.l.noop_search_st(
+            search_root.encode(ls.charset),
+            scope,
+            filterstr=filterstr2.encode(ls.charset),
+            timeout=SEARCH_NOOP_TIMEOUT,
+          )
+        except LDAPLimitErrors,e:
+          pass
     except (ldap.FILTER_ERROR,ldap.INAPPROPRIATE_MATCHING),e:
       # Give the user a chance to edit his bad search filter
       w2lapp.searchform.w2l_SearchForm(
@@ -513,7 +519,7 @@ def w2l_Search(sid,outf,command,form,ls,dn,connLDAPUrl):
             )
           )
           if num_all_search_results!=None:
-            max_result_msg = '(of %d) ' % (num_all_search_results)            
+            max_result_msg = '(of %d) ' % (num_all_search_results)
 
         result_message = 'Results %d - %d %sof %s search below %s with filter &quot;%s&quot;.' % (
             search_resminindex+1,
@@ -692,14 +698,7 @@ def w2l_Search(sid,outf,command,form,ls,dn,connLDAPUrl):
               tableentry_attrs = template_attrs.intersection(entry.data.keys())
               if tableentry_attrs:
                 # Output entry with the help of pre-defined templates
-                tableentry = msbase.CaseinsensitiveStringKeyDict(default='')
-                for attr in tableentry_attrs:
-                  tableentry[attr] = []
-                  for value in entry[attr]:
-                    tableentry[attr].append(
-                      w2lapp.gui.DataStr(sid,form,ls,dn,sub_schema,attr,value,commandbutton=0)
-                    )
-                  tableentry[attr] = ', '.join(tableentry[attr])
+                tableentry = w2lapp.read.DisplayEntry(sid,form,ls,dn,sub_schema,entry,'searchSep',0)
                 tdlist = []
                 for oc in tdtemplate_oc:
                   tdlist.append(search_tdtemplate[oc] % tableentry)
@@ -772,8 +771,12 @@ def w2l_Search(sid,outf,command,form,ls,dn,connLDAPUrl):
 
             command_table.append(form.applAnchor(
                 'search','Down',sid,
-                [('dn',dn),('scope',str(ldap.SCOPE_ONELEVEL))],
-                title=u'\r\n'.join(down_title_list)
+                (
+                  ('dn',dn),
+                  ('scope',w2lapp.searchform.SEARCH_SCOPE_STR_ONELEVEL),
+                  ('filterstr',u'(objectClass=*)'),
+                ),
+                title=u'\r\n'.join(down_title_list),
             ))
 
         else:
